#include "audiowidget.h"

AudioWidget *AudioWidget::_instance = 0;

AudioWidget *AudioWidget::getInstance() {

    if (_instance == 0) {
        _instance = new AudioWidget();
        Q_CHECK_PTR(_instance);
    }
    return _instance;
}

AudioWidget::AudioWidget()
{
    layout = new QHBoxLayout();
    for (int i= 0; i < Settings::getInstance()->getLayersNumber(); i++ ) {
        layout->insertWidget(i, new AudioLayerWidget(this, tr("Layer %1").arg(i)));
     }
    setLayout(layout);
//    qDebug( "Init AudioWidget");
}

void AudioWidget::mediaLoaded(int layer, QString media)
{
    QLayoutItem * const item = layout->itemAt(layer);
    dynamic_cast<AudioLayerWidget *>(item->widget())->loadMedia(media);
//    qDebug() << "AudioWidget::mediaLoaded Received layer: " << layer
//             << "Media: " << media;
}

void AudioWidget::volChanged(int layer, qreal vol) {
    QLayoutItem * const item = layout->itemAt(layer);
    dynamic_cast<AudioLayerWidget *>(item->widget())->setVol(vol);
//        qDebug() << "AudioWidget::volChanged Received layer: " << layer
//             << "Vol : " << vol;

}

void AudioWidget::playbackChanged(int layer, Status status)
{
    QLayoutItem * const item = layout->itemAt(layer);
    switch (status) {
    case Playing:
      dynamic_cast<AudioLayerWidget *>(item->widget())->play();
      break;
    case Stopped:
      dynamic_cast<AudioLayerWidget *>(item->widget())->stop();
      break;
    case Paused:
      dynamic_cast<AudioLayerWidget *>(item->widget())->pause();
      break;
    }
}
/*
void AudioWidget::layerReceived(int layer)
{
    QLayoutItem * const item = layout->itemAt(layer);
    dynamic_cast<AudioLayerWidget *>(item->widget())->updateWatchDMX(true);
}*/

